#include <Eigen/Core>
#include <Eigen/LU> 
#include <iostream>

int main() {
	// We can create a new vector in 2 dimensions:
	// Here the 2d is for 2 and double (denoting the precision)
	Eigen::Vector2d vector2d(0.5, 0.2);

	// We can also print this directly:
	std::cout << "vector2d = \n" << vector2d  << std::endl;

	std::cout << std::endl;

	// We can also access the vector components directly:
	std::cout << "vector2d[0] = " << vector2d[0] << std::endl;
	std::cout << "vector2d[1] = " << vector2d[1] << std::endl;

	std::cout << std::endl;
	// We can make a 2x2 matrix:
	Eigen::Matrix2d matrix2x2;
	// And then we assign it values
	matrix2x2 << 0.5, 0.3, 0.1, 0.5;

	std::cout << "matrix2x2 = \n" << matrix2x2 << std::endl;

	// And we can access the elements directly
	std::cout << "matrix2x2[0][0] = " << matrix2x2(0, 0) << std::endl;
	std::cout << "matrix2x2[0][1] = " << matrix2x2(0, 1) << std::endl;
	std::cout << "matrix2x2[1][0] = " << matrix2x2(1, 0) << std::endl;
	std::cout << "matrix2x2[1][1] = " << matrix2x2(1, 1) << std::endl;
	
	std::cout << std::endl;
	// We can also compute the inverse of the matrix (do NOT use this for
	// solving linear systems Ax = b)
	Eigen::Matrix2d inverseMatrix2x2(matrix2x2.inverse());

	std::cout << "inverseMatrix2x2 = \n " << inverseMatrix2x2 << std::endl << std::endl;

	// And we can multiply the matrix with another matrix:
	Eigen::Matrix2d identityFromInverse = inverseMatrix2x2 * matrix2x2;
	std::cout << "identityFromInverse = \n" << identityFromInverse << std::endl;

	std::cout << std::endl;

	// And we can multiply matrices with vectors and do ordinary arithmetics:
	Eigen::Vector2d mvMinusMInvV = matrix2x2 * vector2d - inverseMatrix2x2 * vector2d;
	std::cout << "mvMinusMInvV = \n" << mvMinusMInvV << std::endl << std::endl;

	// We can also solve the system matrix2x2 x = vector2d;
	Eigen::Vector2d  x = matrix2x2.lu().solve(vector2d);
	std::cout << "x = \n" << x << std::endl << std::endl;

	// We can also make larger vectors
	Eigen::Vector4d vector4d(1.2, 2.3, 4.4, 5.5);
	std::cout << "vector4d = \n" << vector4d << std::endl << std::endl;
	// And larger matrices
	Eigen::Matrix4d matrix4x4;
	matrix4x4 << 1, 2, 3, 4, 
		     5, 6, 7, 8, 
		     9, 10, 11, 12,
		     13, 14, 15, 16;

	std::cout << "matrix4x4 = \n" << matrix4x4 << std::endl;

	// We can also update the contents of the vector
	vector4d << 0, 1, 2, 3;

	std::cout << "vector4d = \n" << vector4d << std::endl << std::endl;

	// And we can compute the norm
	std::cout << "vector4d.norm() = " << vector4d.norm() << std::endl;
}
